// Generated by CoffeeScript 1.7.1
(function() {
  define(['browser-source-map-support'], function(sourceMapSupport) {
    var e, foo;
    sourceMapSupport.install();
    foo = function() {
      throw new Error('foo');
    };
    try {
      return foo();
    } catch (_error) {
      e = _error;
      if (/\bscript\.coffee\b/.test(e.stack)) {
        return document.body.appendChild(document.createTextNode('Test passed'));
      } else {
        document.body.appendChild(document.createTextNode('Test failed'));
        return console.log(e.stack);
      }
    }
  });

}).call(this);

//# sourceMappingURL=script.map
