require('../../modules/es6.string.from-code-point');
require('../../modules/es6.string.raw');
require('../../modules/es6.string.trim');
require('../../modules/es6.string.iterator');
require('../../modules/es6.string.code-point-at');
require('../../modules/es6.string.ends-with');
require('../../modules/es6.string.includes');
require('../../modules/es6.string.repeat');
require('../../modules/es6.string.starts-with');
require('../../modules/es6.regexp.match');
require('../../modules/es6.regexp.replace');
require('../../modules/es6.regexp.search');
require('../../modules/es6.regexp.split');
require('../../modules/es7.string.at');
require('../../modules/es7.string.pad-left');
require('../../modules/es7.string.pad-right');
require('../../modules/es7.string.trim-left');
require('../../modules/es7.string.trim-right');
require('../../modules/core.string.escape-html');
require('../../modules/core.string.unescape-html');
module.exports = require('../../modules/$.core').String;